#include "testlib.h"
#include <string>
#include <vector>
#include <cmath>

using namespace std;

const long double eps = 0.01;
string t;

long double get_num(string num) {
	int cnt = 0, rem = num.length();
	if (!(num[0] >= '0' && num[0] <= '9')) return -1;
	for (int i = 0; i < num.length(); i++) {
		if (num[i] == '.') {
			cnt++;
			if (cnt > 1) return -1;
			if (cnt) rem = i;
		}
		else if (!(num[i] >= '0' && num[i] <= '9')) return -1;
	}
	
	if (rem == num.length() - 1) return -1;
	
	long double ret = 0.0;
	if (cnt) {
		long double mult = 0.1;
		for (int i = rem + 1; i < num.length(); i++) {
			ret += mult * (num[i] - '0');
			mult *= 0.1;
		}
	}
	long double mult = 1.0;
	for (int i = rem - 1; i >= 0; i--) {
		ret += mult * (num[i] - '0');
		mult *= 10;
	}
	
	return ret;
}

vector <pair <bool, pair <long double, long double>>> parse(InStream& stream, int n) {
    stream.maxMessageLength = 10000000;
	string s = stream.readString();
	int len = s.length();
	t = s;
	
	int cnt = 0, j = 0, g = 0;
	vector <pair <bool, pair <long double, long double>>> par(n);
	par[g].first = true;
	if (s[j] == '-')
		par[g].first = false, j++;
	while (j < len) {
		if (s[j] != '|') stream.quitf(_pe, "1Output format is wrong at %d symbol %c", j, s[j]);
		j++, cnt++;
		
		if (j == len) stream.quitf(_pe, "2Output format is wrong at %d", j);
		if (s[j] == '-')
			par[g].second.first = -1, j++;
		else if (s[j] >= '0' && s[j] <= '9')
			par[g].second.first = 1;
		else stream.quitf(_pe, "3Output format is wrong at %d symbol %c", j, s[j]);
		
		string num = "";
		while (j < len && s[j] != 'x')
			num += s[j++];
		
		if (j == len) stream.quitf(_pe, "4Output format is wrong at %d", j);
		
		long double fir = get_num(num);
		if (fir < 0) stream.quitf(_pe, "5Output format is wrong at %d symbol %c", j, s[j]);
		par[g].second.first *= fir;
		j++;
		
		if (s[j] == '+') par[g].second.second = 1;
		else if (s[j] == '-') par[g].second.second = -1;
		else stream.quitf(_pe, "6Output format is wrong at %d symbol %c", j, s[j]);
		
		j++;
		
		num = "";
		while (j < len && s[j] != '|')
			num += s[j++];
		
		if (j == len) stream.quitf(_pe, "7Output format is wrong at %d", j);
		
		long double sec = get_num(num);
		if (sec < 0) stream.quitf(_pe, "8Output format is wrong at %d symbol %c", j, s[j]);
		par[g].second.second *= sec;
		j++, cnt++;
		
		g++;
		if (cnt == 2 * n && j < len) stream.quitf(_pe, "11Output format is wrong at %d", j);
		if (j == len && cnt != 2 * n) stream.quitf(_pe, "9Output format is wrong at %d", j);
		if (j < len) {
			if (s[j] == '+') par[g].first = true;
			else if (s[j] == '-') par[g].first = false;
			else stream.quitf(_pe, "10Output format is wrong at %d symbol %c", j, s[j]);
			j++;
		}
	}
	
	return par;
}

bool ON2_check(int n, vector <pair <int, int>>& dots, vector <pair <bool, pair <long double, long double>>>& par, InStream& stream) {
	for (int i = 0; i < n; i++) {
		long double x = dots[i].first;
		long double y1 = dots[i].second;
		
		long double y2 = 0.0;
		for (int k = 0; k < n; k++) {
			long double z = par[k].second.first * x + par[k].second.second;
			if (par[k].first) y2 += abs(z);
			else y2 -= abs(z);
		}
		
		if (abs(y1 - y2) - eps > 0)
			stream.quitf(_wa, "1Result is wrong at %.12Lf, %.12Lf != %.12Lf", x, y1, y2);
		
		if (i > 0) {
			x = ((long double)dots[i].first + (long double)dots[i - 1].first) / 2.0;
			y1 = ((long double)dots[i].second + (long double)dots[i - 1].second) / 2.0;
			
			y2 = 0.0;
			for (int k = 0; k < n; k++) {
				long double z = par[k].second.first * x + par[k].second.second;
				if (par[k].first) y2 += abs(z);
				else y2 -= abs(z);
			}
			
			if (abs(y1 - y2) - eps > 0)
				stream.quitf(_wa, "2Result is wrong at %.12Lf, %.12Lf != %.12Lf", x, y1, y2);
		}
	}
	
	return true;
}

bool pred(const pair <long double, long double>& p1, const pair <long double, long double>& p2) {
	return (p1.first - p2.first) < -eps;
}

bool ONlogN_check(int n, vector <pair <int, int>>& dots, vector <pair <bool, pair <long double, long double>>>& par, InStream& stream) {
	vector <pair <long double, long double>> iz;
	for (int i = 0; i < n; i++) {
		if (abs(par[i].second.first) < eps) continue;
		if (par[i].first)
			iz.push_back(make_pair(-(par[i].second.second / par[i].second.first), abs(par[i].second.first)));
		else
			iz.push_back(make_pair(-(par[i].second.second / par[i].second.first), -abs(par[i].second.first)));
	}
	
	sort(iz.begin(), iz.end(), pred);
	
	long double strd = 0.0;
	for (int i = 0; i < iz.size(); i++)
		strd -= iz[i].second;
	
	int j = 0;
	long double y2 = 0.0, x = 0.0, y1 = 0.0, xp = 0.0;
	for (int i = 0; i < n; i++) {
		if (i > 0) {
			x = ((long double)dots[i].first + (long double)dots[i - 1].first) / 2.0;
			y1 = ((long double)dots[i].second + (long double)dots[i - 1].second) / 2.0;
			
			while (j < iz.size()) {
			    if (i == 1 && iz[j].first - x < -eps) {
			        strd += 2 * iz[j].second;
			        j++;
			        continue;
			    }
				if (iz[j].first - x > eps) {
					y2 += (x - xp) * strd;
					break;
				}
				else {
					y2 += (iz[j].first - xp) * strd;
					strd += 2 * iz[j].second;
					xp = iz[j].first;
					j++;
				}
			}
			
			if (j == iz.size())
				y2 += (x - xp) * strd;
			
			xp = x;
			if (abs(y1 - y2) - eps > 0)
				stream.quitf(_wa, "2Result is wrong at %.2Lf, %.2Lf != %.2Lf\n%s", x, y1, y2, t.c_str());
		}
		
		x = dots[i].first;
		y1 = dots[i].second;
		
		if (i == 0) {
			for (int k = 0; k < n; k++) {
				long double z = par[k].second.first * x + par[k].second.second;
				if (par[k].first) y2 += abs(z);
				else y2 -= abs(z);
			}
		}
		else {
			while (j < iz.size()) {
				if (iz[j].first - x > eps) {
					y2 += (x - xp) * strd;
					break;
				}
				else {
					y2 += (iz[j].first - xp) * strd;
					strd += 2 * iz[j].second;
					xp = iz[j].first;
					j++;
				}
			}
			
			if (j == iz.size())
				y2 += (x - xp) * strd;
		}
		
		xp = x;
		if (abs(y1 - y2) - eps > 0)
			stream.quitf(_wa, "1Result is wrong at %.6Lf, %.6Lf != %.6Lf ", x, y1, y2);
	}
	
	return true;
}

int main(int argc, char * argv[]) {
	registerTestlibCmd(argc, argv);
	
	int n = inf.readInt();
	vector <pair <int, int>> dots(n);
	vector <long double> dif(n - 1);
	for (int i = 0; i < n; i++) {
		dots[i].first = inf.readInt();
		dots[i].second = inf.readInt();
		
		if (i) dif[i - 1] = (dots[i].second - dots[i - 1].second) / (long double) (dots[i].first - dots[i - 1].first);
	}
	
	auto par1 = parse(ans, n);
	auto par2 = parse(ouf, n);
	
	/*placeholder check*/
	
	//ON2_check(n, dots, par1, ans);
	//if (ON2_check(n, dots, par2, ouf)) quitf(_ok, "OK");
	
	/*real check*/
	
	ONlogN_check(n, dots, par1, ans);
	if (ONlogN_check(n, dots, par2, ouf)) quitf(_ok, "OK");
	
	return 0;
}